#include "hal_data.h"
#include "mtr_main.h"

#include "RealityAI.h"
#include "finger_friction_combined_model.h"

FSP_CPP_HEADER
void R_BSP_WarmStart(bsp_warm_start_event_t event);
FSP_CPP_FOOTER

#define SMOOTH_LEN 7
#define USE_SHIPPER 0

static uint32_t g_events = 0;
static uint32_t g_err_count = 0;
static bool g_dc_callback = false;
static rai_data_collector_callback_args_t g_callback_args;
rai_data_collector_frame_buffer_t g_frame_buf[8];

extern float g_f4_phase_err_monitor;
//extern float g_f4_iu_ad_monitor;
//extern uint32_t g_rai_count_monitor;

static int predBuff[SMOOTH_LEN];
static int pred = 0;

void rai_data_collector0_callback(const rai_data_collector_callback_args_t *p_args)
{
    g_callback_args.frame_buf_len = p_args->frame_buf_len;
    g_callback_args.frames = p_args->frames;
    g_callback_args.instance_id = p_args->instance_id;
    for(uint8_t i = 0; i < p_args->frames; i++)
    {
        g_frame_buf[i] = p_args->p_frame_buf[i];
    }
    g_callback_args.p_frame_buf = g_frame_buf;

    g_dc_callback = true;
}

void rai_data_collector0_error_callback(const rai_data_collector_error_callback_args_t *p_args)
{
    g_events |= p_args->event;
}

void rai_data_collector_error_callback(const rai_data_collector_error_callback_args_t *p_args)
{
    g_events |= p_args->event;
}

void rai_data_shipper0_callback(rai_data_shipper_callback_args_t *p_args)
{
    if(p_args->result != FSP_SUCCESS)
    {
        g_err_count++;
    }
    g_rai_data_collector0.p_api->bufferRelease(&g_rai_data_collector0_ctrl);
}

/*******************************************************************************************************************//**
 * main() is generated by the RA Configuration editor and is used to generate threads if an RTOS is used.  This function
 * is called by main() when no RTOS is used.
 **********************************************************************************************************************/
void hal_entry(void)
{
    fsp_err_t err = FSP_SUCCESS;
    /* TODO: add your own code here */
    mtr_init();

    err = RM_RAI_DATA_SHIPPER_Open(&g_rai_data_shipper0_ctrl, &g_rai_data_shipper0_cfg);

    err = RM_RAI_DATA_COLLECTOR_Open(&g_rai_data_collector0_ctrl, &g_rai_data_collector0_cfg);
    err = RM_RAI_DATA_COLLECTOR_SnapshotChannelRegister(&g_rai_data_collector0_ctrl, 0, &g_f4_phase_err_monitor);

    err = RM_RAI_DATA_COLLECTOR_SnapshotStart(&g_rai_data_collector0_ctrl);

    /* Main Routine */
    while(1)
    {
        mtr_main();

        if(g_dc_callback){
            rai_data_shipper_write_params_t arg;
            arg.diagnostic_data_len = 0;
            arg.events = g_events;
            arg.p_diagnostic_data = NULL;
            arg.p_sensor_data = &g_callback_args;
            g_dc_callback = false;
#if( USE_SHIPPER )
            err = RM_RAI_DATA_SHIPPER_Write(&g_rai_data_shipper0_ctrl, &arg);
#else
            pred = finger_friction_combined_predict(arg.p_sensor_data->p_frame_buf->p_buf);

            predBuff[SMOOTH_LEN - 1] = pred;

            for(int i = 0; i < SMOOTH_LEN - 1; i++)
                predBuff[i] = predBuff[i + 1];

            int cnt = 0, ucnt = 0;
            for(int i = 0; i < SMOOTH_LEN; i++){
                if(predBuff[i] == finger_friction_combined_friction)
                    ucnt++;
                else if(predBuff[i] == finger_friction_combined_normal)
                    cnt++;
            }

            if(ucnt >> cnt)
                pred = finger_friction_combined_friction;
            else
                pred = finger_friction_combined_normal;

            switch(pred){
                case finger_friction_combined_friction:
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED1, MTR_LED_ON);
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED2, MTR_LED_ON);
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED3, MTR_LED_ON);
                    break;
                case finger_friction_combined_normal:
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED1, MTR_LED_OFF);
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED2, MTR_LED_OFF);
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED3, MTR_LED_OFF);
                    break;
                case finger_friction_combined_no_results:
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED1, MTR_LED_OFF);
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED2, MTR_LED_ON);
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED3, MTR_LED_OFF);
                    break;
                default:
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED1, MTR_LED_OFF);
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED2, MTR_LED_ON);
                    R_IOPORT_PinWrite(&g_ioport_ctrl, MTR_PORT_LED3, MTR_LED_OFF);
                    break;
            }
            g_rai_data_collector0.p_api->bufferRelease(&g_rai_data_collector0_ctrl);
#endif
        }
    }
    RM_RAI_DATA_COLLECTOR_SnapshotStop(&g_rai_data_collector0_ctrl);

#if BSP_TZ_SECURE_BUILD
    /* Enter non-secure code */
    R_BSP_NonSecureEnter();
#endif
}

/*******************************************************************************************************************//**
 * This function is called at various points during the startup process.  This implementation uses the event that is
 * called right before main() to set up the pins.
 *
 * @param[in]  event    Where at in the start up process the code is currently at
 **********************************************************************************************************************/
void R_BSP_WarmStart(bsp_warm_start_event_t event)
{
    if (BSP_WARM_START_RESET == event)
    {
#if BSP_FEATURE_FLASH_LP_VERSION != 0

        /* Enable reading from data flash. */
        R_FACI_LP->DFLCTL = 1U;

        /* Would normally have to wait tDSTOP(6us) for data flash recovery. Placing the enable here, before clock and
         * C runtime initialization, should negate the need for a delay since the initialization will typically take more than 6us. */
#endif
    }

    if (BSP_WARM_START_POST_C == event)
    {
        /* C runtime environment and system clocks are setup. */

        /* Configure pins. */
        R_IOPORT_Open (&g_ioport_ctrl, g_ioport.p_cfg);
    }
}

#if BSP_TZ_SECURE_BUILD

BSP_CMSE_NONSECURE_ENTRY void template_nonsecure_callable ();

/* Trustzone Secure Projects require at least one nonsecure callable function in order to build (Remove this if it is not required to build). */
BSP_CMSE_NONSECURE_ENTRY void template_nonsecure_callable ()
{

}
#endif
